# GetLearnerPreds.pl takes one or more .sum files as its input, and collects
# the predicted ratings, etc. for the wug verbs tested in the English Wug Test
# experiment, ready to merge into, for example, the Exp2Ratings.txt file
#
# the reason that this file is useful is that it weeds out duplicate and
# unnecessary outputs, keeping only the best way to derive each desired output
#
# the desired outputs are kept hardwired in a hash table, initialized in a subroutine

initarrays();

print "Choose an option:\n\t(1) process individual file\n\t(2) process .bat file\n? ";
$mode = <STDIN>;
chomp($mode);

while (!$stop)
{
    if ($mode == 1) {
	# ask user to enter the filename and process it
	print "Please enter name of .sum file: ";
	$inputfile = <STDIN>;
	readfile();
	
	$stop = true;    
    } elsif ($mode == 2) {
	# ask user to enter the .bat filename and process it
	print "Please enter name of batch file: ";
	$batchfile = <STDIN>;
	readbatch();
	$stop = true;    
    } else {
	print "Sorry, I couldn't understand you.  Please try again.\n? ";    
    }   
}
print ("Enter name for output file: ");
$outputfile = <STDIN>;
chomp($outputfile);
open( OUTPUTFILE, ">$outputfile") or die "Can't open output file: $!\n";


# first write the header line
printf OUTPUTFILE "number\tverb\treg past\t";

# we'll output just the predicted ratings on the left, then fuller infos on the right
foreach $input (sort keys %inputs) {
    printf OUTPUTFILE "$input reg rating\t";
}
printf OUTPUTFILE "\tirreg past\t";
foreach $input (sort keys %inputs) {
    printf OUTPUTFILE "$input irreg rating\t";
}

#now the fuller infos
printf OUTPUTFILE "\tnumber\tverb\treg past\t";

# we'll list the reg infos first
foreach $input (sort keys %inputs) {
    printf OUTPUTFILE "$input reg rating\t$input reg island\t$input reg hits\t$input reg scope\t$input reg impugnor\t";
}
# a blank column before the irregs
printf OUTPUTFILE "\tirreg past\t";
# and then the irreg infos
foreach $input (sort keys %inputs) {
    printf OUTPUTFILE "$input irreg rating\t$input irreg island\t$input irreg hits\t$input irreg scope\t$input irreg impugnor\t";
}
printf OUTPUTFILE "\r";

# now go through the verbs and output the info, in order
foreach $number (sort keys %verb_order )
{
    if ($verb_order{$number} ne "")
    {        
	$verb = $verb_order{$number};
	$reg_past = $verb_pasts{$number}[0];
	$irreg_past = $verb_pasts{$number}[1];    
	
	# first just the ratings, for doing correlations
	printf OUTPUTFILE "$number\t$verb\t$reg_past\t";
	# first the regs
	foreach $input (sort keys %inputs) {
	    printf OUTPUTFILE "$ratings{$input}{$reg_past}\t";	    
	}
	# the blank column
	printf OUTPUTFILE "\t$irreg_past\t";
	
	# and the irreg numbers
	foreach $input (sort keys %inputs) {
	    if ($ratings{$input}{$irreg_past} == "")
	    {
		$ratings{$input}{$irreg_past} = 0;		
	    }
	    
	    printf OUTPUTFILE "$ratings{$input}{$irreg_past}\t";	    
	}

	
	# and now the fuller infos
	printf OUTPUTFILE "\t$number\t$verb\t$reg_past\t";
	# first the regs
	foreach $input (sort keys %inputs) {
	    printf OUTPUTFILE "$ratings{$input}{$reg_past}\t$islands{$input}{$reg_past}\t$hits{$input}{$reg_past}\t$scope{$input}{$reg_past}\t$impugners{$input}{$reg_past}\t";
	}
	# the blank column
	printf OUTPUTFILE "\t$irreg_past\t";
	
	# and the irreg numbers
	foreach $input (sort keys %inputs) {
	    if ($ratings{$input}{$irreg_past} == "")
	    {
		$ratings{$input}{$irreg_past} = 0;		
	    }
	    
	    printf OUTPUTFILE "$ratings{$input}{$irreg_past}\t$islands{$input}{$irreg_past}\t$hits{$input}{$irreg_past}\t$scope{$input}{$irreg_past}\t$impugners{$input}{$irreg_past}\t";
	}
		
	printf OUTPUTFILE "\r";
    } else {
	printf OUTPUTFILE "null\r";
    }
    
    
}



close(OUTPUTFILE);
MacPerl::SetFileInfo("XCEL", "TEXT", $outputfile);
# end of main routine


# now some subroutines
sub initarrays {

    # verbs to get keeps all the verbs we need    
    $verbs_to_get =  " grEl/grEld lQn/lQnd mn/mnd Sn/Snd skOl/skOld skEl/skEld skwIl/skwIld snEl/snEld kIv/kIvd lm/lmd pm/pmd Si/Sid ze/zed bYz/bYzd dYz/dYzd drYs/drYst flIJ/flIJd fro/frod ger/gerd glIp/glIpt rYf/rYft stIn/stInd stIp/stIpt blIg/blIgd Cek/Cekt drIt/drItd flip/flipt glid/glidd glIt/glItd kwid/kwidd plIm/plImd skrYd/skrYdd splIN/splINd tip/tipt gud/gudd nN/nNd pQNk/pQNkt prik/prikt rQsk/rQskt SIlk/SIlkt tark/tarkt tNk/tNkt trIsk/trIskt blef/bleft brEJ/brEJd Cul/Culd dep/dept gEz/gEzd nes/nest spQk/spQkt stYr/stYrd tES/tESt wIs/wIst nold/noldd flEt/flEtd grYnt/grYntd rYnt/rYntd SYnt/SYntd CYnd/CYndd grEl/grElt lQn/lQnt mn/mnt Sn/Snt skOl/skOlt skEl/skElt skwIl/skwIlt snEl/snElt snEl/snold kIv/kev lm/lem pm/pem Si/Sȍ ze/zEd bYz/boz dYz/doz drYs/dros flIJ/flJ fro/fru ger/gor glIp/glp rYf/rof stIn/stn stIp/stp blIg/blg Cek/CUk drIt/drIt flip/flEpt glid/glEd glIt/glIt kwid/kwEd plIm/plm skrYd/skrod splIN/splN tip/tEpt gud/gud nold/nEld nN/nQN pQNk/pNk prik/prok rQsk/rsk SIlk/SQlk tark/tork tNk/tQNk trIsk/trQsk blef/blEft brEJ/broJ Cul/Col dep/dQpt gEz/gaz nes/nos spQk/spk stYr/stor tES/taS wIs/ws nold/nold rYf/rIf stIn/stQn drIt/drQt glIt/glQt plIm/plQm skrYd/skrId splIN/splQN prik/prEk trIsk/trsk CYnd/CWnd flEt/flEt grYnt/gront rYnt/ront SYnt/Sont CYnd/CYnd grYnt/grWnt rYnt/rWnt SYnt/SWnt ";
    # the list that keeps track of the order of verbs we need
     %verb_order = (1 => "grEl",   2 => "lQn",   3 => "mn",   4 => "Sn",   
 	5 => "skOl",   6 => "skEl",   7 => "skwIl",   8 => "snEl",   
 	9 => "snEl",   10 => "kIv",   11 => "lm",   12 => "pm",   
 	13 => "Si",   14 => "ze",   15 => "bYz",   16 => "dYz",   
 	17 => "drYs",   18 => "flIJ",   19 => "fro",   20 => "ger",   
 	21 => "glIp",   22 => "rYf",   23 => "stIn",   24 => "stIp",   
 	25 => "blIg",   26 => "Cek",   27 => "drIt",   28 => "flip",   
 	29 => "glid",   30 => "glIt",   31 => "kwid",   32 => "plIm",   
 	33 => "skrYd",   34 => "splIN",   35 => "tip",   36 => "gud",   
 	37 => "nold",   38 => "nN",   39 => "pQNk",   40 => "prik",   
 	41 => "rQsk",   42 => "SIlk",   43 => "tark",   44 => "tNk",   
 	45 => "trIsk",   46 => "blef",   47 => "brEJ",   48 => "Cul",   
 	49 => "dep",   50 => "gEz",   52 => "nes",   53 => "spQk",   
 	54 => "stYr",   55 => "tES",   56 => "wIs",   57 => "nold",   
 	58 => "rYf",   59 => "stIn",   60 => "drIt",   61 => "glIt",   
 	62 => "plIm",   63 => "skrYd",   64 => "splIN",   65 => "prik",   
 	66 => "trIsk",   67 => "CYnd",   68 => "flEt",   69 => "grYnt",   
 	70 => "rYnt",   71 => "SYnt",   72 => "CYnd",   73 => "grYnt",   
 	74 => "rYnt",   75 => "SYnt", 76 => "glid" );

    %verb_pasts = (1 => ["grEld", "grElt"], 2 => ["lQnd", "lQnt"], 3 => ["mnd", "mnt"], 
	4 => ["Snd", "Snt"], 5 => ["skOld", "skOlt"], 6 => ["skEld", "skElt"], 
	7 => ["skwIld", "skwIlt"], 8 => ["snEld", "snElt"], 9 => ["", "snold"], 
	10 => ["kIvd", "kev"], 11 => ["lmd", "lem"], 12 => ["pmd", "pem"], 
	13 => ["Sid", "Sȍ"], 14 => ["zed", "zEd"], 15 => ["bYzd", "boz"], 
	16 => ["dYzd", "doz"], 17 => ["drYst", "dros"], 18 => ["flIJd", "flJ"], 
	19 => ["frod", "fru"], 20 => ["gerd", "gor"], 21 => ["glIpt", "glp"], 
	22 => ["rYft", "rof"], 23 => ["stInd", "stn"], 24 => ["stIpt", "stp"], 
	25 => ["blIgd", "blg"], 26 => ["Cekt", "CUk"], 27 => ["drItd", "drIt"], 
	28 => ["flipt", "flEpt"], 29 => ["glidd", "glEd"], 30 => ["glItd", "glIt"], 
	31 => ["kwidd", "kwEd"], 32 => ["plImd", "plm"], 33 => ["skrYdd", "skrod"], 
	34 => ["splINd", "splN"], 35 => ["tipt", "tEpt"], 36 => ["gudd", "gud"], 
	37 => ["", "nEld"], 38 => ["nNd", "nQN"], 39 => ["pQNkt", "pNk"], 
	40 => ["prikt", "prok"], 41 => ["rQskt", "rsk"], 42 => ["SIlkt", "SQlk"], 
	43 => ["tarkt", "tork"], 44 => ["tNkt", "tQNk"], 45 => ["trIskt", "trQsk"], 
	46 => ["bleft", "blEft"], 47 => ["brEJd", "broJ"], 48 => ["Culd", "Col"], 
	49 => ["dept", "dQpt"], 50 => ["gEzd", "gaz"], 52 => ["nest", "nos"], 
	53 => ["spQkt", "spk"], 54 => ["stYrd", "stor"], 55 => ["tESt", "taS"], 
	56 => ["wIst", "ws"], 57 => ["noldd", "nold"], 58 => ["", "rIf"], 
	59 => ["", "stQn"], 60 => ["", "drQt"], 61 => ["", "glQt"], 
	62 => ["", "plQm"], 63 => ["", "skrId"], 64 => ["", "splQN"], 
	65 => ["", "prEk"], 66 => ["", "trsk"], 67 => ["", "CWnd"], 
	68 => ["flEtd", "flEt"], 69 => ["grYntd", "gront"], 70 => ["rYntd", "ront"], 
	71 => ["SYntd", "Sont"], 72 => ["CYndd", "CYnd"], 73 => ["", "grWnt"], 
	74 => ["", "rWnt"], 75 => ["", "SWnt"], 76 => ["", "glid"],  );
  
}
sub readfile {
    chomp($inputfile);
    $input = $inputfile;
    $input =~ s/.sum$//;
    $inputs{$input} = 1;    

    open( INPUTFILE, "<$inputfile") or die "Can't open input file $inputfile: $!\n";
    print "reading input file: $inputfile\n";
    
    # the first line is the headers -- ignore it     
    $line = <INPUTFILE>;
    
    # now comes the juicy part
    while ($line = <INPUTFILE>)
    {
        chomp($line);
	# each line contains a whole mess o' fields
	($form, $pattern, $output, $mcat1, undef, $mcat2, undef, $A, undef, $B, $Change, 
	  undef, $Pres, $Pfeat, $P, undef, $Q, $Qfeat, $Qres, $doppel, $scope, $typefreq, 
	  $tokenfreq, $rel_tokenfreq, $overall, $wbf, $wbl, $c75, $c90, $c95, $i75, $i90, 
	  $i95, $constraint, $related_forms, $exceptions, $impugner, $iScope, $iHits, $impugner_info, $impugner_constraint, @rest)
	   = split ("\t", $line);
	  
	$currentverb = "$mcat1/$mcat2";
	$replace = 0;
	
	if ( $verbs_to_get =~ /\s$currentverb\s/s)
	{
	    if ($overall > $ratings{$input}{$mcat2})
	    {
	        $replace = true;
	    } elsif ( $overall eq $ratings{$input}{$mcat2}) {
	        if ($B eq "d")
		{
		    $replace = true;
		} 
		
	    }
	    
	    if ( $replace )
	    {		
	        $ratings{$input}{$mcat2} = $overall;
		$islands{$input}{$mcat2} = "$A -> $B / $Pres $Pfeat $P ___ $Q $Qfeat $Qres";
		$impugners{$input}{$mcat2} = $impugner_constraint;
		$hits{$input}{$mcat2} = $typefreq;
		$scope{$input}{$mcat2} = $scope;
#		print "Getting info for $currentverb\n";		
	    }
	} else {
#	    print "$currentverb isn't one we're looking for\n";
	}
    }
    
    close(INPUTFILE);
}

sub readbatch()
{
    chomp ($batchfile);
    open (BATCHFILE, "<$batchfile") or die "Can't open batch file: $!\n";
    
    while ($line = <BATCHFILE>)
    {	
	chomp($line);	
	($sumfile, $c, $i) = split( "\t", $line);
	$sumfile =~ s/.in$//;
	$inputfile = $sumfile . "_c" . $c . "i" . $i . ".sum";		
	readfile();	
    }    
}
